-------------------------------------------------------------------------------
--! @file       TweakeyUpdate.vhd
--! @author     Ekawat (ice) Homsirikamol
--! @brief      TweakeyUpdate
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use work.AES_pkg.all;

entity TweakeyUpdate is
    generic (
        G_NO_TWEAKEYS   : integer := 2
    );
    port    (
        ii              : in  std_logic_vector(128*G_NO_TWEAKEYS           -1 downto 0);
        oo              : out std_logic_vector(128*G_NO_TWEAKEYS           -1 downto 0)
    );
end entity TweakeyUpdate;

architecture structure of TweakeyUpdate is
    type perm_type is array(0 to 15) of integer;
    constant PERM : perm_type := (1,6,11,12,5,10,15,0,9,14,3,4,13,2,7,8);

    type state_type is array (0 to G_NO_TWEAKEYS-1, 0 to 15) of std_logic_vector(7 downto 0);

    signal tw : state_type;
    signal hh : state_type;
    signal gg : state_type;
begin
    gWord:
    for i in 0 to G_NO_TWEAKEYS-1 generate
        gLoop:
        for j in 0 to 15 generate
            tw(i,j)       <= ii(128*(G_NO_TWEAKEYS-i)-j*8-1 downto 128*(G_NO_TWEAKEYS-i)-(j+1)*8);
            hh(i,PERM(j)) <= tw(i,j);

            g2: if (G_NO_TWEAKEYS = 2) generate
                gg1: if i = 0 generate
                    m2: entity work.AES_mul(AES_mulx02)
                        port map (input => hh(i,j), output => gg(i,j));
                end generate;
                gg2: if i = 1 generate
                    gg(i,j)   <= hh(i,j);
                end generate;
            end generate;
            g3: if (G_NO_TWEAKEYS = 3) generate
                gg1: if i = 0 generate
                    m4: entity work.AES_mul(AES_mulx04)
                        port map (input => hh(i,j), output => gg(i,j));
                end generate;
                gg2: if i = 1 generate
                    m2: entity work.AES_mul(AES_mulx02)
                        port map (input => hh(i,j), output => gg(i,j));
                end generate;
                gg4: if i = 2 generate
                    gg(i,j)   <= hh(i,j);
                end generate;
            end generate;

            oo(128*(G_NO_TWEAKEYS-i)-j*8-1 downto 128*(G_NO_TWEAKEYS-i)-(j+1)*8) <= gg(i,j);
        end generate;
    end generate;
end architecture structure;
